package com.cultural.heritage.controller.good;


import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.FileConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.entity.Festival;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 节日管理接口
 */
@RestController
@RequestMapping("/festival")
@Slf4j
@Tag(name = "节日管理模块")
public class FestivalController {


    @Resource(name = "festivalRedisTemplate")
    private RedisTemplate<String, Festival> redisTemplate;

    // 节日集合的键
    private static final String FESTIVAL_KEY = "festivalList";


    /**
     * Web端管理员添加节日项
     * @param festival 节日添加信息
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加节日项", description = "参数：节日添加信息，权限：管理员(admin, boss)，方法名：addFestivalElement")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addFestivalElement(@RequestBody Festival festival) {
        if (festival == null || StringUtils.isAnyBlank(festival.getName(), festival.getUrl())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "存在参数为空");
        }
        if (isElementExistInList(festival.getName())) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项已存在");
        }
        redisTemplate.opsForList().rightPush(FESTIVAL_KEY, festival);
        return ResultUtils.success(true);
    }


    /**
     * Web端管理员根据节日名称获取图片URL
     * @param festivalName  节日名称
     * @return  当前节日的url
     */
    @GetMapping("/getUrl")
    @Operation(summary = "Web端管理员根据节日名称获取URL", description = "参数：节日名称，权限：管理员(admin, boss)，方法名：getFestivalUrl")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<String> getFestivalUrl(@RequestParam String festivalName) {
        // 获取 Redis 中所有的节日数据
        List<Festival> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        if (festivalList == null || festivalList.isEmpty()) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "节日列表为空");
        }

        // 遍历查找匹配的节日
        for (Festival festival : festivalList) {
            if (festival.getName().equals(festivalName)) {
                // TODO
                String wholeFilePath = FileConstant.SERVER_HOST + festival.getUrl();
                return ResultUtils.success(wholeFilePath); // 返回对应的 URL
            }
        }
        throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项不存在");
    }



    /**
     * Web端管理员查询节日集合
     * @return 节日列表
     */
    @GetMapping("/get")
    @Operation(summary = "Web端管理员查询节日集合", description = "参数：无，权限：管理员(admin, boss)，方法名：getFestivalList")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<Festival>> getFestivalList() {
        List<Festival> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        // TODO
        for (Festival festival : festivalList) {
            festival.setUrl(FileConstant.SERVER_HOST + festival.getUrl());
        }
        return ResultUtils.success(festivalList);
    }


    /**
     * Web端管理员删除节日项
     * @param festivalName 删除的节日项
     * @return 是否删除成功
     */
    @GetMapping("/delete")
    @Operation(summary = "Web端管理员删除节日项", description = "参数：无，权限：管理员(admin, boss)，方法名：deleteFestivalElement")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteFestivalElement(@RequestParam String festivalName) {
        List<Festival> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        if (festivalList == null || festivalList.isEmpty()) {
            throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项不存在");
        }
        // 遍历列表，找到匹配的对象并删除
        for (Festival festival : festivalList) {
            if (festival.getName().equals(festivalName)) {
                redisTemplate.opsForList().remove(FESTIVAL_KEY, 1, festival);
                return ResultUtils.success(true);
            }
        }
        throw new BusinessException(ErrorCode.OPERATION_ERROR, "该节日项不存在");
    }



    // 检查元素是否存在
    private boolean isElementExistInList(String festivalName) {
        List<Festival> festivalList = redisTemplate.opsForList().range(FESTIVAL_KEY, 0, -1);
        if (festivalList == null) return false;

        for (Festival festival : festivalList) {
            if (festival.getName().equals(festivalName)) {
                return true;
            }
        }
        return false;
    }


}
